<?php
/**
 * @file
 * All the node functions being used for the webtexttool module.
 */

/**
 * Implements hook_form_FORM_ID_alter().
 */
function webtexttool_form_node_form_alter(&$form, &$form_state, $form_id) {
  $node = $form['#node'];

  // We only show the tool when user has access
  // and it is enabled on node type form.
  if (user_access('webtexttool_use')
    && variable_get('webtexttool_enabled_' . $node->type, 0) == 1) {

    $form['#attached']['css'] = array(
      drupal_get_path('module', 'webtexttool') . '/css/webtexttool.css',
    );
    $form['#attached']['js'] = array(
      drupal_get_path('module', 'webtexttool') . '/js/webtexttool.js',
    );

    $form['webtexttool'] = array(
      '#type' => 'fieldset',
      '#title' => t('Webtexttool'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#weight' => 35,
      '#group' => 'additional_settings',
      '#attributes' => array(
        'class' => array('webtexttool-form'),
      ),
      '#attached' => array(
        'js' => array(
          'vertical-tabs' => drupal_get_path('module', 'webtexttool') . '/js/webtexttool_vertical_tabs.js',
        ),
      ),
    );
    $form['webtexttool']['analyse_page'] = array(
      '#type' => 'button',
      '#value' => t('Analyse page'),
      '#title' => t('Get keyword suggestion'),
      '#ajax' => array(
        'callback' => 'webtexttool_analyse_form_node_callback',
        'wrapper' => 'webtexttool-analyse',
        'effect' => 'fade',
        'event' => 'click',
      ),
    );
    $form['webtexttool']['webtexttool_keywords'] = array(
      '#type' => 'textfield',
      '#title' => t('Keyword'),
      '#default_value' => isset($node->webtexttool_keywords) ? $node->webtexttool_keywords : '',
      '#description' => t('Enter the keywords which should be analyzed.'),
      '#maxlength' => 512,
    );
    $form['webtexttool']['webtexttool_language'] = array(
      '#type' => 'select',
      '#title' => t('Keyword suggestions language'),
      '#options' => webtexttool_getkeywordsources(),
      '#description' => t('Based on the keyword we can lookup synonyms that can be added in the text to score even higher in search engines. This languague will be used to lookup synonyms.'),
      '#default_value' => isset($node->webtexttool_language) ? $node->webtexttool_language : '',
    );
    $form['webtexttool']['get_keyword_suggestion'] = array(
      '#type' => 'button',
      '#value' => t('Get keyword suggestion'),
      '#title' => t('Get keyword suggestion'),
      '#ajax' => array(
        'callback' => 'webtexttool_suggestion_form_node_callback',
        'wrapper' => 'webtexttool-keyword-suggestion',
        'effect' => 'fade',
        'event' => 'click',
      ),
    );
    $form['webtexttool']['keyword_container'] = array(
      '#prefix' => '<div id="webtexttool-keyword-suggestion">',
      '#suffix' => '</div>',
    );
    // Added a pure css loading.
    // See http://codepen.io/agrimsrud/pen/vtmoq
    $form['webtexttool_analyse_container'] = array(
      '#prefix' => '<div id="webtexttool-analyse">' . t('Loading Webtexttool SEO optimisation tool') . '<br /><br /><div class="loader"><i></i><i></i><i></i><i></i><i></i></div>',
      '#suffix' => '</div>',
    );

  }
}

/**
 * Returns changed part of the form.
 *
 * @return array
 *   Form API array.
 *
 * @see ajax_example_form_node_form_alter()
 */
function webtexttool_suggestion_form_node_callback($form, &$form_state) {
  $values = $form_state['values'];
  $keywords_value = $values['webtexttool_keywords'];
  $database = $values['webtexttool_language'];
  $keywords = explode(',', $keywords_value);
  global $language;



  $header = array(
    t("Keyword"),
    t("Search volume"),
    t("Competition"),
    t("Overall"),
  );

  $rows = array();

  foreach ($keywords as $keyword) {
    $response = webtexttool_searchkeyword($keyword, $database, $language->language);
    if ($response && is_array($response)) {
      foreach ($response as $suggested_keyword) {
        $keyword = $suggested_keyword['Keyword'];
        $volumescore = $suggested_keyword['VolumeScore'];
        $competitionscore = $suggested_keyword['CompetitionScore'];
        $overallscore = $suggested_keyword['OverallScore'];
        $volumeindicator = webtexttool_color_indicator(webtexttool_score_to_class($volumescore));
        $competitionindicator = webtexttool_color_indicator(webtexttool_score_to_class($competitionscore));
        $overallindicator  = webtexttool_color_indicator(webtexttool_overall_score_to_class($overallscore));

        $rows[] = array(
          $keyword,
          $volumeindicator . webtexttool_voilume_score_to_text($volumescore),
          $competitionindicator . webtexttool_competition_score_to_text($competitionscore),
          $overallindicator . webtexttool_overall_to_text($overallscore),
        );
      }
    }
  }

  $output = theme('table', array('header' => $header, 'rows' => $rows));

  $commands[] = ajax_command_html('#webtexttool-keyword-suggestion', $output);
  return array('#type' => 'ajax', '#commands' => $commands);
}

/**
 * AJAX callback for the webtexttool button.
 */
function webtexttool_analyse_form_node_callback($form, &$form_state) {

  $commands = array();

  $node = node_form_submit_build_node($form, $form_state);

  $can_login = FALSE;
  $is_authenticated = _webtexttool_is_authenticated();
  if (!$is_authenticated) {
    $login = _webtexttool_login();
    if ($login) {
      $can_login = TRUE;
    }
  }
  else {
    $can_login = TRUE;
  }
  if (!$can_login) {
    // Unable to login credentials should be checked.
    $output = "<img class='logo' src='" . file_create_url(drupal_get_path("module", "webtexttool") . "/img/wtt_logo.png") . "'>";
    $output .= "<div class='info'>";
    $output .= t('Unable to login with the credentials you filled in at !url', array('!url' => l(t('webtexttool acount form'), 'admin/config/services/webtexttool')));
    $output .= "</div>";
  }
  elseif ($node->webtexttool_keywords == '') {
    // Make sure we have a keyword to check.
    $output = "<img class='logo' src='" . file_create_url(drupal_get_path("module", "webtexttool") . "/img/wtt_logo.png") . "'>";
    $output .= "<div class='info'>";
    $output .= t('Please, enter a keyword in the Webtexttool-tab to get started.');
    $output .= "</div>";
  }
  else {
    // Check if we have a valid nid.
    $node->nid = !empty($node->nid) ? $node->nid : 0;
    if (module_exists('metatag')) {

      // First we need to get the metatags defaults.
      $default_metatags = $form['metatags'][$node->language]['#metatag_defaults'];

      // Now we merge the defaults with the changes in the form.
      foreach ($default_metatags as $key => $metatag) {

        if (!isset($node->metatags[$node->language][$key])) {
          // Set a default value for a metatag das is using the default.
          $node->metatags[$node->language][$key]['value'] = $metatag['value'];

        }
      }
      // Replace all tokens wit the correct value.
      foreach ($node->metatags[$node->language] as $key => $metatag) {
        $node->metatags[$node->language][$key]['value'] = token_replace($node->metatags[$node->language][$key]['value'], array('node' => $node));
      }

    }

    // Now we are going to post to the new menu item.
    $token = drupal_random_key();
    $data = 'token=' . $token . '&node=' . drupal_json_encode($node);
    variable_set('webtexttool_token', $token);

    $options = array(
      'method' => 'POST',
      'data' => $data ,
      'headers' => array(
        'Content-Type' => 'application/x-www-form-urlencoded',
      ),
    );

    // Adding authorisation to the call when the drupal site is.
    if (variable_get('webtexttool_authenticate', 0) == 1 && variable_get('webtexttool_authenticate_credentials', '') != '') {
      $options['headers']['Authorization'] = 'Basic ' . base64_encode(variable_get('webtexttool_authenticate_credentials', ''));
    }

    $url = url('webtexttool/view', array('absolute' => TRUE));
    $request = drupal_http_request($url, $options);
    $output = "";

    // Check response code.
    if ($request->code == 200 && $request->status_message == 'OK') {
      $response = webtexttool_analyse($request->data, $node->webtexttool_keywords, $node->webtexttool_language);
      if ($response) {
        $output = theme('webtexttool-suggestion', $response);
      }
    }
    else {
      $output .= "<div class='info'>";
      $output .= t('Unable to load html of this node. Please check if an anonymous user can access a full rendered node of this type.');
      $output .= "</div>";
    }

  }

  $commands[] = ajax_command_html('#webtexttool-analyse', $output);
  $commands[] = ajax_command_remove('div.messages');
  $commands[] = ajax_command_html('#console', theme('status_messages'));
  return array('#type' => 'ajax', '#commands' => $commands);
}

/**
 * Implements hook_node_load().
 */
function webtexttool_node_load($nodes, $types) {
  $vids = array();
  foreach ($nodes as $node) {
    $vids[$node->vid] = $node->vid;
  }
  $result = db_query('SELECT nid, keywords, lang FROM {webtexttool} WHERE vid IN(:vids)', array(':vids' => array_keys($vids)));
  foreach ($result as $record) {
    $nodes[$record->nid]->webtexttool_keywords = $record->keywords;
    $nodes[$record->nid]->webtexttool_language = $record->lang;
  }
}

/**
 * Implements hook_node_insert().
 */
function webtexttool_node_insert($node) {
  // Only insert into database if we need to exclude the node.
  if (!empty($node->webtexttool_keywords) || !empty($node->webtexttool_language)) {
    db_insert('webtexttool')
      ->fields(array(
        'nid' => $node->nid,
        'vid' => $node->vid,
        'keywords' => $node->webtexttool_keywords ? $node->webtexttool_keywords : '',
        'lang' => $node->webtexttool_language ? $node->webtexttool_language : '',
      ))
      ->execute();
  }
}

/**
 * Implements hook_node_update().
 */
function webtexttool_node_update($node) {
  // If the user created a new revision will we insert a new record in the db.
  // Else update database if we need to exclude the node.
  // Otherwise we delete the record from the database.
  if (!$node->is_new && !empty($node->revision) && $node->vid) {
    webtexttool_node_insert($node);
  }
  elseif (!empty($node->webtexttool_keywords) || !empty($node->webtexttool_language)) {
    db_merge('webtexttool')
      ->key(array('vid' => $node->vid))
      ->fields(array(
        'nid' => $node->nid,
        'keywords' => $node->webtexttool_keywords ? $node->webtexttool_keywords : '',
        'lang' => $node->webtexttool_language ? $node->webtexttool_language : '',
      ))
      ->execute();
  }
  else {
    webtexttool_node_revision_delete($node);
  }
}

/**
 * Implements hook_node_delete().
 */
function webtexttool_node_delete($node) {
  db_delete('webtexttool')
    ->condition('nid', $node->nid)
    ->execute();
}

/**
 * Implements hook_node_revision_delete().
 */
function webtexttool_node_revision_delete($node) {
  db_delete('webtexttool')
    ->condition('vid', $node->vid)
    ->execute();
}

/**
 * Implements hook_form_FORM_ID_alter().
 */
function webtexttool_form_node_type_form_alter(&$form, &$form_state) {
  if (isset($form['type'])) {
    $form['webtexttool'] = array(
      '#type' => 'fieldset',
      '#title' => t('Webtexttool settings'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#group' => 'additional_settings',
      '#attributes' => array(
        'class' => array('webtexttool-node-type-settings-form'),
      ),
      '#attached' => array(
        'js' => array(drupal_get_path('module', 'webtexttool') . '/js/webtexttool_node_type.js'),
      ),
    );

    $form['webtexttool']['webtexttool_enabled'] = array(
      '#type' => 'checkbox',
      '#title' => t('Enabled'),
      '#default_value' => variable_get('webtexttool_enabled_' . $form['#node_type']->type, 0),
      '#description' => t('Enable the webtexttool for this contenttype.'),
    );
  }
}

/**
 * Function to render a node based on a post from the form.
 */
function webtexttool_view() {
  $current_token = variable_get('webtexttool_token', '');
  if (isset($_POST) && isset($_POST['token'])) {
    if ($_POST['token'] != $current_token) {
      drupal_not_found();
      drupal_exit();
    }
  }
  else {
    drupal_not_found();
    drupal_exit();
  }
  // First we need to get the post data.
  if (isset($_POST) && isset($_POST['node'])) {
    $node = (object) drupal_json_decode($_POST['node']);
  }
  else {
    drupal_not_found();
    drupal_exit();
  }

  // Getting globals.
  global $user;

  drupal_page_is_cacheable(FALSE);
  // Make a copy of the current user and set new current user as anonymous.
  $current_user = clone $user;
  $user = drupal_anonymous_user();

  // Re-initialize path.
  $_GET['q'] = 'node';
  if ($_GET['q'] == 'node' && !empty($node->nid)) {
    $_GET['q'] .= '/' . $node->nid;
  }

  // Render all metatags by hand.
  $head = '';
  if (isset($node->metatags) && isset($node->metatags[$node->language])) {
    foreach ($node->metatags[$node->language] as $meta_key => $meta_value) {
      if (isset($meta_value['value'])) {
        if ($meta_key == 'title') {
          $head .= '<title>' . strip_tags($meta_value['value']) . '</title>';
        }
        else {
          $head .= '<meta name="' . check_plain($meta_key) . '" content="' . strip_tags($meta_value['value']) . '" /> ';
        }
      }
    }
  }
  // Render the custom <meta name="url" content="">.
  $path = '';
  if (module_exists('path') && $node->path['alias'] != '') {
    $path = $node->path['alias'];
  }

  if (module_exists('pathauto') && $node->path['pathauto'] == 1) {
    $pattern = pathauto_pattern_load_by_entity('node', $node->type, $node->language);
    $path = token_replace($pattern, array('node' => $node));
  }
  if ($path != '') {
    $path = url($path);
    $head .= '<meta name="url" content="' . $path . '" /> ';
  }
  // Get a very plain version of the html.
  $page = '<html><head>' . $head;
  $page .= '</head><body>';

  // Render node.
  $node_view = node_view($node);
  $key = empty($node->nid) ? $node->nid : 0;
  $nodes['nodes'][$key] = $node_view;

  // Set page content, page title and render page.
  drupal_set_title(check_plain($node->title));
  drupal_set_page_content($node_view);
  $page .= drupal_render($node_view);
  $page .= '</body></html>';

  // Change back user object.
  $user = $current_user;
  print $page;
}
